#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
set UNIQUE_SWITCHBOX_SIZE 0
# Nanometers to microns conversion function
proc um { val } { return [expr $val / 1000.0] }
proc snap_to_grid_X { val pitch} {return [expr int($val / $pitch) *$pitch] }



set TOP "Top_PipelinedCipher"
#set CLKNET "CONF_CLK"
#set RSTNET "rstn"
set GNDNET "gnd"
set VDDNET "vdd"
set M 4
set N 4
set NETLIST_DIR .
#sourcing to set global variables
source gen_globals.tcl
# Sourcing the configuration:
unset argv ;
source aes.globals ;
set_message_verbosity TECHLIB-359 off; # ST .lib are badly parsed by encounter
init_design -setup typ_setup -hold typ_setup
set_analysis_view -setup typ_setup \
		  -hold  typ_setup
fit


proc my_info { msg } \
{ \
  set l "Info \[pr\]:"
  puts "$l $msg"
}

#-------------------------------------------------------------------------------
# Additional configuration
#-------------------------------------------------------------------------------

# Units in nanometer:
set PITCH_NM       200 ;
set CELL_HEIGHT_NM [ expr 13 * $PITCH_NM ] ;
# Units in micron:
set PITCH          [ expr       $PITCH_NM / 1e3 ] ;
set CELL_HEIGHT    [ expr $CELL_HEIGHT_NM / 1e3 ] ;

if {$TOP == "FPGA"} {
set SUPPLY_TO_DIE  [ expr        10 *$PITCH ] ;
set SUPPLY_WIDTH   [ expr        10 * $PITCH ] ;
set SUPPLY_TO_CORE [ expr        8 * $PITCH ] ;
} else {
set SUPPLY_TO_DIE  [ expr        10* $PITCH ] ; 
set SUPPLY_WIDTH   [ expr        10 * $PITCH ] ;
set SUPPLY_TO_CORE [ expr        8 * $PITCH ] ;
}
#<-(4.92-6*pitch) that makes ~2.46, 2 is the width of core row feeds+allowance for HCMOS9GP
set CORE_TO_LEFT   [ expr $SUPPLY_TO_DIE + $SUPPLY_WIDTH + $SUPPLY_TO_CORE ] ;
set CORE_TO_RIGHT  [ expr $SUPPLY_TO_DIE + $SUPPLY_WIDTH + $SUPPLY_TO_CORE ] ;
set CORE_TO_BOTTOM [ expr $SUPPLY_TO_DIE + $SUPPLY_WIDTH + $SUPPLY_TO_CORE ] ;
set CORE_TO_TOP    [ expr $SUPPLY_TO_DIE + $SUPPLY_WIDTH + $SUPPLY_TO_CORE ] ;

# The ratio and the density are the values after the duplication:
set RATIO   1.0 ;
set DENSITY 0.50 ; 
set SBOX_WIDTH 44.2;
set SBOX_HEIGHT 41.6;
set HALF_SBOX_LENGTH 46.43;
set HALF_SBOX_WIDTH 42.64;
set SBOX_SPACING 4;
set ALLOWANCE 5*$PITCH;
set STD_CELL_HEIGHT 13*$PITCH;
set RING_WIDTH 0.4;
set RING_SPACE 0.2;
#-------------------------------------------------------------------------------
# Floorplanning
#-------------------------------------------------------------------------------
#i changed it from f to s
setFlipping f 
setBottomIoPadOrient R0

floorPlan -site CORE12T -r [ expr ${RATIO} / 1 ] ${DENSITY} \
  ${CORE_TO_LEFT} ${CORE_TO_BOTTOM} ${CORE_TO_RIGHT} ${CORE_TO_TOP}
saveFPlan ${TOP}.fp
legalizeFPlan  -checkSite
checkFPlan -place
# Floorplan information:
my_info "Floorplan ended successfully.";

#-------------------------------------------------------------------------------
# Power routing
#-------------------------------------------------------------------------------

# Done now (before placement) otherwise it interferes with place/route constraints
connectToGlobalNet                    \
  -tie_highs_lows 1                   \
  -to_global_net gnd                  \
  -override_prior_global_connection 0 \
  -in_instances *                     \
  -nets {}                            \
  -pins gnd                           \
  -connect Pins                       \
  -under_module {}                    ;
connectToGlobalNet                    \
  -tie_highs_lows 1                   \
  -to_global_net vdd                  \
  -verbose 0                          \
  -override_prior_global_connection 0 \
  -in_instances * -nets {}            \
  -pins vdd                           \
  -connect Pins                       \
  -under_module {}                    ;
# To bind the 1'b0 and 1'b1 to the ground/power net:
globalNetConnect gnd -type tielo
globalNetConnect vdd -type tiehi

}

sroute  -nets {vdd gnd}  -noStripes -noPadRings -noBlockPins \
-straightConnections { straightWithDRCClean straightWithChanges };





##-------------------------------------------------------------------------------
##Pre Route section
##-------------------------------------------------------------------------------
if { [file exists ${TOP}.route.gz] } {
restoreRoute ${TOP}.route.gz

if {[string equal -nocase -length 4 $TOP "cbox"]} {
set i 0;
while { $i < 8} {
	setAttribute -net chan\[$i\] -skip_routing true;
	incr i;

}
set i 0;
while { $i < 7} {
	setAttribute -net clb_side\[$i\] -skip_routing true;
#	setAttribute -net clb_side1\[$i\] -skip_routing true;
	incr i;

}

}
}

saveDesign ${TOP}_preroute.enc
#-------------------------------------------------------------------------------
#Adding Well taps
#-------------------------------------------------------------------------------


#######addWellTap -cell C12T32_LR_FILLERNPW4 -prefix welltap -maxGap 30


#-------------------------------------------------------------------------------
# Placement
#-------------------------------------------------------------------------------

#The placement obstructions:
if {![string equal -nocase -length 3 $TOP "CT_"]} {
amoebaPlace -slow
if {$TOP == "FPGA"} {
#placePIO -ioFile $ROOT_DIR/safe_array/arch/SAFE.io
}
#checkPlace
}
trialRoute -maxRouteLayer 5
#setExtractRCMode -engine postRoute -effortLevel signoff
#extractRC
#rcOut -spf Top_PipelinedCipher.spf
#rcOut -spef Top_PipelinedCipher.spef

