# Create directory structure 
# ------------------------------------------------------------------------
 sh mkdir -p WORK
 sh mkdir -p DB
 sh mkdir -p REPORTS
 sh mkdir -p RTL
 sh mkdir -p GATES
 set TOP Top_PipelinedCipher

# Set Library
# ------------------------------------------------------------------------
 set LIB C28SOI_SC_12_CORE_LL
 set LIB_VERSION 3.1-00
 #set PVT tt28_0.85V_25C
 set PVT tt28_0.85V_0.00V_0.00V_0.00V_25C
 set PERIOD 0.57
 set LIB_FOLDER cmos028fdsoi_25f
 #set LIB_FOLDER cmos028fdsoi_27a
 #set FRONTEND_VERSION 6.0-00
 set FRONTEND_VERSION 5.2-00

 # Define the location of the default design library
 define_design_lib WORK -path WORK

 # Set up target technology and component libraries 
 set search_path [ list /comelec/stkits/${LIB_FOLDER}/${LIB}/${LIB_VERSION}/libs \
	/comelec/stkits/${LIB_FOLDER}/FrontEndKit/${FRONTEND_VERSION}/etc/GENERIC_WLM_library/ \
	/comelec/stkits/${LIB_FOLDER}/FrontEndKit/${FRONTEND_VERSION}/DC_STAR_SYNTHESIS/CUSTOM_SCRIPTS/ ]
 #set target_library [list C28SOI_SC_12_CORE_LR_tt28_1.20V_25C.db]
 set target_library [list ${LIB}_${PVT}.db]
 set wlm_library [list GENERIC_WLM_LIBRARY.db]
 #set target_library [list C28SOI_SC_12_CORE_LR_tt28_0.70V_m40C.db]
 set synthetic_library [list standard.sldb ]
 set link_library [concat "*" $target_library $synthetic_library $wlm_library]
# ------------------------------------------------------------------------

 set srcs {./src/AddRoundKey.v  ./src/KeyExpantion.v  ./src/MixColumns.v  ./src/RoundKeyGen.v  ./src/Round.v  ./src/SBox.v  ./src/ShiftRows.v  ./src/SubBytes.v  ./src/Top_PipelinedCipher.v}
 analyze -autoread $srcs
 elaborate Top_PipelinedCipher

 write -hierarchy -output DB/elab.db
 # Define the clock and its period 
 create_clock clk -period 5
 compile

 # Write out a Verilog model of the RTL design
 write  -format verilog -hierarchy\
 	  -output RTL/Top_PipelinedCipher.vm
 #write reports
 report_timing > REPORTS/timing.rpt
 report_area > REPORTS/area.rpt
 report_power > REPORTS/power.rpt

start_gui
